% script ESV_tool

%
% Copyright 2018, David W. Eaton, University of Calgary, all rights reserved
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER "AS IS" AND ANY EXPRESS OR 
% IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
% MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
% SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
% INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
% TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
% BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
% STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
% OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

% This program accompanies the textbook "Passive Seismic Monitoring of
% Induced Seismicity: Fundamental Principles and Application to Energy
% Technologies", Cambridge University Press
%
% Refer to the online solutions manual for usage information

disp('Warning: Your workspace will be cleared. Hit enter to continue.');
pause;

clear all;
close all;


fname = uigetfile('*.mat','Input event file');

load(fname);

[K,V] = convhull(xfrac,yfrac,zfrac);

for i = 1:length(xfrac);
    v = [mean(xfrac)-xfrac(i),mean(yfrac)-yfrac(i),mean(zfrac)-zfrac(i)];
    v = v/norm(v);
    
    xfracmin(i) = xfrac(i) + errfrac(i)*v(1);
    yfracmin(i) = yfrac(i) + errfrac(i)*v(2);
    zfracmin(i) = xfrac(i) + errfrac(i)*v(3);
    
    xfracmax(i) = xfrac(i) - errfrac(i)*v(1);
    yfracmax(i) = yfrac(i) - errfrac(i)*v(2);
    zfracmax(i) = xfrac(i) - errfrac(i)*v(3);
    
end

[Kmin,Vmin] = convhull(xfracmin,yfracmin,zfracmin);
[Kmax,Vmax] = convhull(xfracmax,yfracmax,zfracmax);


% Now generate mirror points

for i = 1:length(xfrac);
    
    xfm(i) = x0 + (x0-xfrac(i));
    yfm(i) = y0 + (y0-yfrac(i));
    zfm(i) = zfrac(i);
    
end

xfm = xfm';
yfm = yfm';
zfm = zfm';

xaug = [xfrac
    xfm];
yaug = [yfrac,
    yfm];
zaug = [zfrac,
    zfm];

efaug = [errfrac,
    errfrac];

[Kmir,Vmir] = convhull(xaug,yaug,zaug);

for i = 1:length(xaug);
    v = [x0-xaug(i),y0-yaug(i),mean(zaug)-zaug(i)];
    v = v/norm(v);
    
    xaugmin(i) = xaug(i) + efaug(i)*v(1);
    yaugmin(i) = yaug(i) + efaug(i)*v(2);
    zaugmin(i) = xaug(i) + efaug(i)*v(3);
    
    xaugmax(i) = xaug(i) - efaug(i)*v(1);
    yaugmax(i) = yaug(i) - efaug(i)*v(2);
    zaugmax(i) = xaug(i) - efaug(i)*v(3);
    
end

[Kamin,Vamin] = convhull(xaugmin,yaugmin,zaugmin);
[Kamax,Vamax] = convhull(xaugmax,yaugmax,zaugmax);

disp('Volume of convex hull (m^3):')
V

disp('Minimum volume (m^3):')
Vmin

disp('Maximum volume (m^3):')
Vmax

subplot(1,2,1), plot3(xfrac,yfrac,zfrac,'k+')
axis image
xlabel('X (m)')
ylabel('Y (m)')
zlabel('Z (m)');

subplot(1,2,2), trisurf(K,xfrac,yfrac,zfrac)
axis image
xlabel('X (m)')
ylabel('Y (m)')
zlabel('Z (m)');





